@@ -19,8 +19,8 @@
 #' @param x input matrix of dimension nobs x nvars.
 #' @param y outcome vector of length nobs.
 #' @param views a matrix of dimension nvars x (levels - 1), where each entry is an integer describing to which view each feature corresponds.
-#' @param type the type of MVS model to be fitted. Currently only type "StaPLR" is supported.
-#' @param levels an integer >= 2, specifying the number of levels in the MVS procedure.
+#' @param type a character vector of length 1 or length \code{levels}, specifying the type(s) of learner to be used at each level of MVS. Use type "StaPLR" when the desired learner(s) is/are penalized GLM(s); see \code{\link[mvs]{StaPLR}} for supported families. Use type "RF" for random forests.
+#' @param levels (optional) an integer >= 2, specifying the number of levels in the MVS procedure. The default is to infer the number of levels from the supplied \code{views} argument.
 #' @param alphas a numeric vector of length \code{levels} specifying the value of the alpha parameter to use at each level.
 #' @param relax either a logical vector of length \code{levels} specifying whether model relaxation (e.g. the relaxed lasso) should be employed at each level, or a single TRUE or FALSE to enable or disable relaxing across all levels. Defaults to FALSE.
 #' @param adaptive either a logical vector of length \code{levels} specifying whether adaptive weights (e.g. the adaptive lasso) should be employed at each level, or a single TRUE or FALSE to enable or disable adaptive weights across all levels. Note that using adaptive weights is generally only sensible if alpha > 0. Defaults to FALSE.
@@ -30,7 +30,7 @@
 #' @param parallel whether to use foreach to fit the learners and obtain the cross-validated predictions at each level in parallel. Executes sequentially unless a parallel back-end is registered beforehand.
 #' @param seeds (optional) a vector specifying the seed to use at each level.
 #' @param progress whether to show a progress bar (only supported when parallel = FALSE).
-#' @param ... additional arguments to pass to the learning algorithm. See e.g. ?StaPLR. Note that these arguments are passed to the the learner at every level of the MVS procedure.
+#' @param ... additional arguments to pass to the learning algorithm. See e.g. \code{\link[mvs]{StaPLR}}. Note that these arguments are passed to the the learner at every level of the MVS procedure.
 #' @return An object of S3 class "MVS".
 #' @keywords TBA
 #' @import foreach
@@ -45,21 +45,24 @@
 #' p <- 1 /(1 + exp(-eta))
 #' y <- rbinom(n, 1, p)
 #'
-#' ## 2-level MVS
+#' ## 2-level MVS with ridge for baselearners and lasso for meta learner
 #' views <- c(rep(1,45), rep(2,20), rep(3,20))
 #' fit <- MVS(x=X, y=y, views=views)
 #' 
+#' ## 2-level MVS with random forest for base learners and lasso for meta learner
+#' fit <- MVS(x=X, y=y, views=views, type = c("RF", "StaPLR"))
+#' new_X <- matrix(rnorm(2*85), nrow=2)
+#' predict(fit, new_X)
+#' 
 #' ## 3-level MVS
 #' bottom_level <- c(rep(1:3, each=15), rep(4:5, each=10), rep(6:9, each=5))
 #' top_level <- c(rep(1,45), rep(2,20), rep(3,20))
 #' views <- cbind(bottom_level, top_level)
 #' fit <- MVS(x=X, y=y, views=views, levels=3, alphas=c(0,1,1), nnc=c(0,1,1))
 #' coefficients <- coef(fit)
-#'
-#' new_X <- matrix(rnorm(2*85), nrow=2)
-#' predict(fit, new_X)}
-
-MVS <- function(x, y, views, type="StaPLR", levels=2, alphas=c(0,1), nnc=c(0,1), parallel=FALSE, 
+#' predict(fit, new_X)
+#' }
+MVS <- function(x, y, views, type="StaPLR", levels=NULL, alphas=c(0,1), nnc=c(0,1), parallel=FALSE, 
                 seeds=NULL, progress=TRUE, relax = FALSE, adaptive = FALSE, na.action = "fail", na.arguments = NULL, ...){
   
   staplr.args <- names(list(...))
@@ -70,8 +73,24 @@ MVS <- function(x, y, views, type="StaPLR", levels=2, alphas=c(0,1), nnc=c(0,1),
     }
   }
   
-  if (levels == 2L) if (!is.matrix(views)) views <- matrix(views, ncol = 1L)
+  if (!is.matrix(views)) views <- matrix(views, ncol = 1L)
   
+  if (is.null(levels)) levels <- ncol(views) + 1L
+  
+  if (levels > 1L && length(type) == 1L) type <- rep(type, times = levels)
+  
+  if (levels != length(alphas)){
+    warning("Length of argument alphas not equal to the number of levels.")
+  }
+  
+  if (levels != length(nnc)){
+    warning("Length of argument nnc not equal to the number of levels.")
+  }
+  
+  if (levels > 2L && na.action == "pass"){
+    warning("na.action = pass, but levels > 2. Passing missingness through multiple levels is not recommended.")
+  }
+    
   if(length(relax)==1){
     relax <- rep(relax, levels)
   }
@@ -88,23 +107,55 @@ MVS <- function(x, y, views, type="StaPLR", levels=2, alphas=c(0,1), nnc=c(0,1),
   }
 
   ## Fit lowest-level baselearners
-  pred_functions[[1]] <- learn(X=x, y=y, views=views[,1], type=type, alpha1 = alphas[1], ll1=ll[1],
-                               seed=seeds[1], progress=progress, parallel=parallel, 
-                               relax.base = relax[1L], penalty.weights.base = translate_adaptive_argument(adaptive[1L]), 
-                               na.action=na.action, na.arguments=na.arguments, ...)
+  arg_list <- list(X = x, 
+                   y=y,
+                   views=views[,1L],
+                   type=type[1L], 
+                   ...)
+  if (type[levels] == "StaPLR") {
+    arg_list <- append(arg_list, list(
+      alpha1=alphas[1L], 
+      ll1=ll[1L],
+      seed=seeds[1L],
+      progress=progress, 
+      parallel=parallel, 
+      relax.base = relax[1L],
+      penalty.weights.base = translate_adaptive_argument(adaptive[1L]),
+      na.action=na.action, 
+      na.arguments=na.arguments))
+  } else if (type[levels] == "RF") {
+    arg_list <- append(arg_list, list(
+      na.action=na.action, 
+      na.arguments=na.arguments))
+  }
+  pred_functions[[1L]] <- do.call(learn, arg_list)
 
   ## Fit intermediate-level learners
   if(levels > 2){
-    for(i in 2:ncol(views)){
-      if(progress){
-        message(paste("Level", i, "\n"))
+    for(i in 2L:ncol(views)){
+      if(progress) message(paste("Level", i, "\n"))
+      arg_list <- list(X = pred_functions[[i-1L]]$CVs, 
+                       y=y,
+                       views=condense(views, level=i),
+                       type=type[i], 
+                       ...)
+      if (type[levels] == "StaPLR") {
+        arg_list <- append(arg_list, list(
+          alpha1=alphas[i], 
+          ll1=ll[i],
+          seed=seeds[i],
+          progress=progress, 
+          parallel=parallel, 
+          relax.base = relax[i],
+          penalty.weights.base = translate_adaptive_argument(adaptive[i]),
+          na.action=na.action, 
+          na.arguments=na.arguments))
+      } else if (type[levels] == "RF") {
+        arg_list <- append(arg_list, list(
+          na.action=na.action, 
+          na.arguments=na.arguments))
       }
-      pred_functions[[i]] <- learn(pred_functions[[i-1]]$CVs, y,
-                                   views=condense(views, level=i), type=type,
-                                   alpha1 = alphas[i], ll1=ll[i], seed=seeds[i],
-                                   progress=progress, parallel=parallel, relax.base = relax[i],
-                                   penalty.weights.base = translate_adaptive_argument(adaptive[i]),
-                                   na.action=na.action, na.arguments=na.arguments,...)
+      pred_functions[[i]] <- do.call(learn, arg_list)
     }
   }
 
@@ -113,18 +164,38 @@ MVS <- function(x, y, views, type="StaPLR", levels=2, alphas=c(0,1), nnc=c(0,1),
   }
 
   ## Fit meta learner
-  pred_functions[[ncol(views)+1]] <- learn(pred_functions[[ncol(views)]]$CVs, y=y,
-                                           views=rep(1,ncol(pred_functions[[ncol(views)]]$CVs)),
-                                           type=type, alpha1=alphas[ncol(views)+1], ll1=ll[ncol(views)+1],
-                                           generate.CVs=FALSE, seed=seeds[ncol(views)+1],
-                                           progress=progress, parallel=parallel, 
-                                           relax.base = relax[ncol(views)+1],
-                                           penalty.weights.base = translate_adaptive_argument(adaptive[ncol(views)+1]),
-                                           na.action=na.action, na.arguments=na.arguments,...)
+  arg_list <- list(X = pred_functions[[ncol(views)]]$CVs, 
+                   y=y,
+                   views=rep(1,ncol(pred_functions[[ncol(views)]]$CVs)),
+                   type=type[levels], 
+                   generate.CVs=FALSE,
+                   ...)
+  if (type[levels] == "StaPLR") {
+    arg_list <- append(arg_list, list(
+      alpha1=alphas[ncol(views)+1], 
+      ll1=ll[ncol(views)+1],
+      seed=seeds[ncol(views)+1],
+      progress=progress, 
+      parallel=parallel, 
+      relax.base = relax[ncol(views)+1],
+      penalty.weights.base = translate_adaptive_argument(adaptive[ncol(views)+1]),
+      na.action=na.action, 
+      na.arguments=na.arguments))
+  } else if (type[levels] == "RF") {
+    arg_list <- append(arg_list, list(
+      na.action=na.action, 
+      na.arguments=na.arguments))
+  }
+  
+  if(arg_list$na.action != "pass"){
+    pred_functions[[ncol(views)+1]] <- do.call(learn, arg_list)
+  }
 
   for(i in 1:length(pred_functions)){
-    pred_functions[[i]]$meta <- NULL
-    names(pred_functions[[i]])[1] <- "models"
+    if(!is.null(pred_functions[[i]])){
+      pred_functions[[i]]$meta <- NULL
+      names(pred_functions[[i]])[1] <- "models"
+    }
   }
 
   names(pred_functions) <- paste("Level", 1:(ncol(views)+1))
@@ -168,10 +239,9 @@ mvs <- MVS
 #'
 #' new_X <- matrix(rnorm(2*85), nrow=2)
 #' predict(fit, new_X)}
-
 predict.MVS <- function(object, newx, predtype = "response", cvlambda = "lambda.min",
                         ...){
-
+  
   x <- newx
 
   for(i in 1:length(object)){
@@ -183,8 +253,16 @@ predict.MVS <- function(object, newx, predtype = "response", cvlambda = "lambda.
     } else pt <- predtype
 
     for(j in 1:ncol(Z)){
-      Z[,j] <- predict(object[[i]]$models[[j]], x[, object[[i]]$view == j, drop=FALSE],
-                       s = cvlambda, type = pt, ...)
+      if (inherits(object[[i]]$models[[j]], "randomForest")) {
+        Z[,j] <- if (is.factor(object[[i]]$models[[j]]$y)) {
+          predict(object[[i]]$models[[j]], x[, object[[i]]$view == j, drop=FALSE], type = "prob", ...)[ , 2L]
+        } else {
+          predict(object[[i]]$models[[j]], x[, object[[i]]$view == j, drop=FALSE], type = "response", ...)
+        }
+      } else if (inherits(object[[i]]$models[[j]], c("cv.glmnet", "glmnet"))) {
+        Z[,j] <- predict(object[[i]]$models[[j]], x[, object[[i]]$view == j, drop=FALSE],
+                        s = cvlambda, type = pt, ...)
+      }
     }
     x <- Z
   }
@@ -200,8 +278,6 @@ predict.MVS <- function(object, newx, predtype = "response", cvlambda = "lambda.
 #' @param object An object of class "MVS".
 #' @param cvlambda By default, the coefficients are extracted at the CV-optimal values of the penalty parameters. Choosing "lambda.1se" will extract them at the largest values within one standard error of the minima.
 #' @param ... Further arguments to be passed to \code{\link[glmnet]{coef.cv.glmnet}}.
-#' 
-#' 
 #' @return An object of S3 class "MVScoef".
 #' @keywords TBA
 #' @export
@@ -234,7 +310,11 @@ coef.MVS <- function(object, cvlambda = "lambda.min", ...){
 
   for(i in 1:length(object)){
     for(j in 1:length(object[[i]]$models)){
-      out[[i]][[j]] <- coef(object[[i]]$models[[j]], s=cvlambda, ...)
+      if(inherits(object[[i]]$models[[j]], "cv.glmnet")){
+        out[[i]][[j]] <- coef(object[[i]]$models[[j]], s=cvlambda, ...)
+      }else{
+        out[[i]][[j]] <- NA
+      }
     }
   }
   names(out) <- paste("Level", 1:length(object))
@@ -243,3 +323,50 @@ coef.MVS <- function(object, cvlambda = "lambda.min", ...){
 
   return(out)
 }
+
+
+#' Calculate feature importance from an "MVS" object.
+#'
+#' Calculate feature importance at each level from an "MVS" object based on random forests.
+#' @param x An object of class "MVS".
+#' @param ... Further arguments to be passed to \code{\link[randomForest]{importance}}.
+#' @return An object of S3 class "MVSimportance".
+#' @keywords TBA
+#' @export
+#' @author Wouter van Loon <w.s.van.loon@fsw.leidenuniv.nl>
+#' @examples \donttest{ 
+#' set.seed(012)
+#' n <- 1000
+#' X <- matrix(rnorm(8500), nrow=n, ncol=85)
+#' views <- c(rep(1,45), rep(2,20), rep(3,20))
+#' beta <- c(rep(10, 55), rep(0, 30)) * ((rbinom(85, 1, 0.5)*2)-1)
+#' eta <- X %*% beta
+#' p <- 1 /(1 + exp(-eta))
+#' y <- rbinom(n, 1, p)
+#' 
+#' ## 2-level MVS with random forest
+#' fit <- MVS(x=X, y=y, views=views, type = "RF")
+#' importance(fit)}
+importance.MVS <- function(x, ...){
+  
+  out <- vector("list", length(x))
+  
+  for(i in 1:length(x)){
+    out[[i]] <- vector("list", length(x[[i]]$models))
+  }
+  
+  for(i in 1:length(x)){
+    for(j in 1:length(x[[i]]$models)){
+      if(inherits(x[[i]]$models[[j]], "randomForest")){
+        out[[i]][[j]] <- randomForest::importance(x[[i]]$models[[j]], ...)
+      }else{
+        out[[i]][[j]] <- NA
+      }
+    }
+  }
+  names(out) <- paste("Level", 1:length(x))
+  attr(out, "type") <- attr(x, "type")
+  class(out) <- "MVSimportance"
+  
+  return(out)
+}
